@@ -28,6 +28,8 @@ from wandb.util import (
 if wandb.TYPE_CHECKING:  # type: ignore
     import typing as t
     from . import summary_record as sr
+    from typing import Optional
+    from wandb.proto.wandb_internal_pb2 import PollExitResponse
 
 logger = logging.getLogger("wandb")
 
@@ -59,7 +61,7 @@ class MessageRouter(object):
             self._object_ready = threading.Event()
             self._lock = threading.Lock()
 
-        def get(self, timeout=None):
+        def get(self, timeout=None) -> Optional[object]:
             is_set = self._object_ready.wait(timeout)
             if is_set and self._object:
                 return self._object
@@ -167,10 +169,13 @@ class BackendSender(object):
         rec = self._make_record(history=history)
         self._publish(rec)
 
-    def publish_history(self, data, step=None, run=None):
+    def publish_history(self, data, step=None, run=None, publish_step=True):
         run = run or self._run
         data = data_types.history_dict_to_json(run, data, step=step)
         history = wandb_internal_pb2.HistoryRecord()
+        if publish_step:
+            history.step.num = step
+        data.pop("_step", None)
         for k, v in six.iteritems(data):
             item = history.item.add()
             item.key = k
@@ -197,6 +202,8 @@ class BackendSender(object):
         proto_artifact.type = artifact.type
         proto_artifact.name = artifact.name
         proto_artifact.digest = artifact.digest
+        if artifact.distributed_id:
+            proto_artifact.distributed_id = artifact.distributed_id
         if artifact.description:
             proto_artifact.description = artifact.description
         if artifact.metadata:
@@ -549,6 +556,7 @@ class BackendSender(object):
         req = self._make_record(run=run)
         resp = self._communicate(req, timeout=timeout)
         if resp is None:
+            logger.info("couldn't get run from backend")
             # Note: timeouts handled by callers: wandb_init.py
             return
         assert resp.run_result
@@ -569,7 +577,13 @@ class BackendSender(object):
         self._publish(rec)
 
     def publish_artifact(
-        self, run, artifact, aliases, is_user_created=False, use_after_commit=False
+        self,
+        run,
+        artifact,
+        aliases,
+        is_user_created=False,
+        use_after_commit=False,
+        finalize=True,
     ):
         proto_run = self._make_run(run)
         proto_artifact = self._make_artifact(artifact)
@@ -578,6 +592,7 @@ class BackendSender(object):
         proto_artifact.entity = proto_run.entity
         proto_artifact.user_created = is_user_created
         proto_artifact.use_after_commit = use_after_commit
+        proto_artifact.finalize = finalize
         for alias in aliases:
             proto_artifact.aliases.append(alias)
         rec = self._make_record(artifact=proto_artifact)
@@ -618,11 +633,15 @@ class BackendSender(object):
         assert result.exit_result
         return result.exit_result
 
-    def communicate_poll_exit(self, timeout=None):
+    def communicate_poll_exit(self, timeout=None) -> Optional[PollExitResponse]:
         poll_request = wandb_internal_pb2.PollExitRequest()
         rec = self._make_request(poll_exit=poll_request)
         result = self._communicate(rec, timeout=timeout)
-        return result
+        if result is None:
+            return None
+        poll_exit_response = result.response.poll_exit_response
+        assert poll_exit_response
+        return poll_exit_response
 
     def communicate_check_version(self, current_version=None):
         check_version = wandb_internal_pb2.CheckVersionRequest()
@@ -635,8 +654,9 @@ class BackendSender(object):
             return
         return result.response.check_version_response
 
-    def communicate_run_start(self):
+    def communicate_run_start(self, run_pb):
         run_start = wandb_internal_pb2.RunStartRequest()
+        run_start.run.CopyFrom(run_pb)
         rec = self._make_request(run_start=run_start)
         result = self._communicate(rec)
         return result
