@@ -54,6 +54,8 @@ class ArtifactSaver(object):
         self,
         type,
         name,
+        distributed_id=None,
+        finalize=True,
         metadata=None,
         description=None,
         aliases=None,
@@ -87,6 +89,7 @@ class ArtifactSaver(object):
             labels=labels,
             description=description,
             is_user_created=self._is_user_created,
+            distributed_id=distributed_id,
         )
 
         # TODO(artifacts):
@@ -111,12 +114,15 @@ class ArtifactSaver(object):
                 'Unknown artifact state "{}"'.format(self._server_artifact["state"])
             )
 
-        self._api.create_artifact_manifest(
-            "wandb_manifest.json",
+        artifact_manifest_id, _ = self._api.create_artifact_manifest(
+            "wandb_manifest.json"
+            if distributed_id is None
+            else "wandb_manifest.patch.json",
             "",
             artifact_id,
             base_artifact_id=latest_artifact_id,
             include_upload=False,
+            type="FULL" if not distributed_id else "PATCH",
         )
 
         step_prepare = wandb.filesync.step_prepare.StepPrepare(
@@ -129,7 +135,11 @@ class ArtifactSaver(object):
             self._manifest,
             artifact_id,
             lambda entry, progress_callback: self._manifest.storage_policy.store_file(
-                artifact_id, entry, step_prepare, progress_callback=progress_callback
+                artifact_id,
+                artifact_manifest_id,
+                entry,
+                step_prepare,
+                progress_callback=progress_callback,
             ),
         )
 
@@ -138,13 +148,26 @@ class ArtifactSaver(object):
                 path = os.path.abspath(fp.name)
                 json.dump(self._manifest.to_manifest_json(), fp, indent=4)
             digest = wandb.util.md5_file(path)
+            if distributed_id:
+                # If we're in the distributed flow, we want to update the
+                # patch manifest we created with our finalized digest.
+                _, resp = self._api.update_artifact_manifest(
+                    artifact_manifest_id, digest=digest,
+                )
+            else:
+                # In the regular flow, we can recreate the full manifest with the
+                # updated digest.
+                #
+                # NOTE: We do this for backwards compatibility with older backends
+                # that don't support the 'updateArtifactManifest' API.
+                _, resp = self._api.create_artifact_manifest(
+                    "wandb_manifest.json",
+                    digest,
+                    artifact_id,
+                    base_artifact_id=latest_artifact_id,
+                )
+
             # We're duplicating the file upload logic a little, which isn't great.
-            resp = self._api.create_artifact_manifest(
-                "wandb_manifest.json",
-                digest,
-                artifact_id,
-                base_artifact_id=latest_artifact_id,
-            )
             upload_url = resp["uploadUrl"]
             upload_headers = resp["uploadHeaders"]
             extra_headers = {}
@@ -155,12 +178,15 @@ class ArtifactSaver(object):
                 self._api.upload_file_retry(upload_url, fp, extra_headers=extra_headers)
 
         def on_commit():
-            if use_after_commit:
+            if finalize and use_after_commit:
                 self._api.use_artifact(artifact_id)
             step_prepare.shutdown()
 
         # This will queue the commit. It will only happen after all the file uploads are done
         self._file_pusher.commit_artifact(
-            artifact_id, before_commit=before_commit, on_commit=on_commit
+            artifact_id,
+            finalize=finalize,
+            before_commit=before_commit,
+            on_commit=on_commit,
         )
         return self._server_artifact
