@@ -10,6 +10,7 @@ import numbers
 import os
 
 import six
+import wandb
 from wandb.proto import wandb_internal_pb2
 
 from . import meta, sample, stats
@@ -17,13 +18,50 @@ from . import tb_watcher
 from ..lib import proto_util
 
 
+if wandb.TYPE_CHECKING:
+    from typing import (
+        Any,
+        Callable,
+        Dict,
+        Iterable,
+        Optional,
+    )
+    from .settings_static import SettingsStatic
+    from six.moves.queue import Queue
+    from threading import Event
+    from ..interface.interface import BackendSender
+    from wandb.proto.wandb_internal_pb2 import Record, Result
+
+    SummaryDict = Dict[str, Any]
+
+
 logger = logging.getLogger(__name__)
 
 
 class HandleManager(object):
+
+    _consolidated_summary: SummaryDict
+    _sampled_history: Dict[str, sample.UniformSampleAccumulator]
+    _settings: SettingsStatic
+    _record_q: "Queue[Record]"
+    _result_q: "Queue[Result]"
+    _stopped: Event
+    _sender_q: "Queue[Record]"
+    _writer_q: "Queue[Record]"
+    _interface: BackendSender
+    _system_stats: Optional[stats.SystemStats]
+    _tb_watcher: Optional[tb_watcher.TBWatcher]
+
     def __init__(
-        self, settings, record_q, result_q, stopped, sender_q, writer_q, interface,
-    ):
+        self,
+        settings: SettingsStatic,
+        record_q: "Queue[Record]",
+        result_q: "Queue[Result]",
+        stopped: Event,
+        sender_q: "Queue[Record]",
+        writer_q: "Queue[Record]",
+        interface: BackendSender,
+    ) -> None:
         self._settings = settings
         self._record_q = record_q
         self._result_q = result_q
@@ -34,35 +72,36 @@ class HandleManager(object):
 
         self._tb_watcher = None
         self._system_stats = None
+        self._step = 0
 
         # keep track of summary from key/val updates
         self._consolidated_summary = dict()
         self._sampled_history = dict()
 
-    def handle(self, record):
+    def handle(self, record: Record) -> None:
         record_type = record.WhichOneof("record_type")
         assert record_type
         handler_str = "handle_" + record_type
-        handler = getattr(self, handler_str, None)
+        handler: Callable[[Record], None] = getattr(self, handler_str, None)
         assert handler, "unknown handle: {}".format(handler_str)
         handler(record)
 
-    def handle_request(self, record):
+    def handle_request(self, record: Record) -> None:
         request_type = record.request.WhichOneof("request_type")
         assert request_type
         handler_str = "handle_request_" + request_type
-        handler = getattr(self, handler_str, None)
+        handler: Callable[[Record], None] = getattr(self, handler_str, None)
         logger.debug("handle_request: {}".format(request_type))
         assert handler, "unknown handle: {}".format(handler_str)
         handler(record)
 
-    def _dispatch_record(self, record, always_send=False):
+    def _dispatch_record(self, record: Record, always_send: bool = False) -> None:
         if not self._settings._offline or always_send:
             self._sender_q.put(record)
         if not record.control.local:
             self._writer_q.put(record)
 
-    def handle_request_defer(self, record):
+    def handle_request_defer(self, record: Record) -> None:
         defer = record.request.defer
         state = defer.state
 
@@ -84,31 +123,31 @@ class HandleManager(object):
         # defer is used to drive the sender finish state machine
         self._dispatch_record(record, always_send=True)
 
-    def handle_request_login(self, record):
+    def handle_request_login(self, record: Record) -> None:
         self._dispatch_record(record)
 
-    def handle_run(self, record):
+    def handle_run(self, record: Record) -> None:
         self._dispatch_record(record)
 
-    def handle_stats(self, record):
+    def handle_stats(self, record: Record) -> None:
         self._dispatch_record(record)
 
-    def handle_config(self, record):
+    def handle_config(self, record: Record) -> None:
         self._dispatch_record(record)
 
-    def handle_output(self, record):
+    def handle_output(self, record: Record) -> None:
         self._dispatch_record(record)
 
-    def handle_files(self, record):
+    def handle_files(self, record: Record) -> None:
         self._dispatch_record(record)
 
-    def handle_artifact(self, record):
+    def handle_artifact(self, record: Record) -> None:
         self._dispatch_record(record)
 
-    def handle_alert(self, record):
+    def handle_alert(self, record: Record) -> None:
         self._dispatch_record(record)
 
-    def _save_summary(self, summary_dict, flush=False):
+    def _save_summary(self, summary_dict: SummaryDict, flush: bool = False) -> None:
         summary = wandb_internal_pb2.SummaryRecord()
         for k, v in six.iteritems(summary_dict):
             update = summary.update.add()
@@ -120,7 +159,7 @@ class HandleManager(object):
         elif not self._settings._offline:
             self._sender_q.put(record)
 
-    def _save_history(self, record):
+    def _save_history(self, record: Record) -> None:
         for item in record.history.item:
             # TODO(jhr) save nested keys?
             k = item.key
@@ -129,14 +168,32 @@ class HandleManager(object):
                 self._sampled_history.setdefault(k, sample.UniformSampleAccumulator())
                 self._sampled_history[k].add(v)
 
-    def handle_history(self, record):
+    def _history_assign_step(self, record: Record, history_dict: Dict) -> None:
+        has_step = record.history.HasField("step")
+        item = record.history.item.add()
+        item.key = "_step"
+        if has_step:
+            step = record.history.step.num
+            history_dict["_step"] = step
+            item.value_json = json.dumps(step)
+            self._step = step + 1
+        else:
+            history_dict["_step"] = self._step
+            item.value_json = json.dumps(self._step)
+            self._step += 1
+
+    def handle_history(self, record: Record) -> None:
+        history_dict = proto_util.dict_from_proto_list(record.history.item)
+        # if syncing an old run, we can skip this logic
+        if history_dict.get("_step") is None:
+            self._history_assign_step(record, history_dict)
+
         self._dispatch_record(record)
         self._save_history(record)
-        history_dict = proto_util.dict_from_proto_list(record.history.item)
         self._consolidated_summary.update(history_dict)
         self._save_summary(self._consolidated_summary)
 
-    def handle_summary(self, record):
+    def handle_summary(self, record: Record) -> None:
         summary = record.summary
 
         for item in summary.update:
@@ -179,64 +236,66 @@ class HandleManager(object):
 
         self._save_summary(self._consolidated_summary)
 
-    def handle_exit(self, record):
+    def handle_exit(self, record: Record) -> None:
         self._dispatch_record(record, always_send=True)
 
-    def handle_final(self, record):
+    def handle_final(self, record: Record) -> None:
         self._dispatch_record(record, always_send=True)
 
-    def handle_header(self, record):
+    def handle_header(self, record: Record) -> None:
         self._dispatch_record(record)
 
-    def handle_footer(self, record):
+    def handle_footer(self, record: Record) -> None:
         self._dispatch_record(record)
 
-    def handle_request_check_version(self, record):
+    def handle_request_check_version(self, record: Record) -> None:
         self._dispatch_record(record)
 
-    def handle_telemetry(self, record):
+    def handle_telemetry(self, record: Record) -> None:
         self._dispatch_record(record)
 
-    def handle_request_run_start(self, record):
+    def handle_request_run_start(self, record: Record) -> None:
         run_start = record.request.run_start
         assert run_start
         assert run_start.run
 
         if not self._settings._disable_stats:
             pid = os.getpid()
-            self._system_stats = stats.SystemStats(pid=pid, interface=self._interface,)
+            self._system_stats = stats.SystemStats(pid=pid, interface=self._interface)
             self._system_stats.start()
 
         if not self._settings._disable_meta:
-            run_meta = meta.Meta(settings=self._settings, interface=self._interface,)
+            run_meta = meta.Meta(settings=self._settings, interface=self._interface)
             run_meta.probe()
             run_meta.write()
 
         self._tb_watcher = tb_watcher.TBWatcher(
-            self._settings, interface=self._interface, run_proto=run_start.run,
+            self._settings, interface=self._interface, run_proto=run_start.run
         )
 
+        if run_start.run.resumed:
+            self._step = run_start.run.starting_step
         result = wandb_internal_pb2.Result(uuid=record.uuid)
         self._result_q.put(result)
 
-    def handle_request_resume(self, data):
+    def handle_request_resume(self, record: Record) -> None:
         if self._system_stats is not None:
             logger.info("starting system metrics thread")
             self._system_stats.start()
 
-    def handle_request_pause(self, data):
+    def handle_request_pause(self, record: Record) -> None:
         if self._system_stats is not None:
             logger.info("stopping system metrics thread")
             self._system_stats.shutdown()
 
-    def handle_request_poll_exit(self, record):
+    def handle_request_poll_exit(self, record: Record) -> None:
         self._dispatch_record(record, always_send=True)
 
-    def handle_request_status(self, record):
+    def handle_request_status(self, record: Record) -> None:
         self._dispatch_record(record)
 
-    def handle_request_get_summary(self, data):
-        result = wandb_internal_pb2.Result(uuid=data.uuid)
+    def handle_request_get_summary(self, record: Record) -> None:
+        result = wandb_internal_pb2.Result(uuid=record.uuid)
         for key, value in six.iteritems(self._consolidated_summary):
             item = wandb_internal_pb2.SummaryItem()
             item.key = key
@@ -244,19 +303,19 @@ class HandleManager(object):
             result.response.get_summary_response.item.append(item)
         self._result_q.put(result)
 
-    def handle_tbrecord(self, record):
+    def handle_tbrecord(self, record: Record) -> None:
         logger.info("handling tbrecord: %s", record)
         if self._tb_watcher:
             tbrecord = record.tbrecord
             self._tb_watcher.add(tbrecord.log_dir, tbrecord.save, tbrecord.root_dir)
         self._dispatch_record(record)
 
-    def handle_request_sampled_history(self, data):
-        result = wandb_internal_pb2.Result(uuid=data.uuid)
+    def handle_request_sampled_history(self, record: Record) -> None:
+        result = wandb_internal_pb2.Result(uuid=record.uuid)
         for key, sampled in six.iteritems(self._sampled_history):
             item = wandb_internal_pb2.SampledHistoryItem()
             item.key = key
-            values = sampled.get()
+            values: Iterable[Any] = sampled.get()
             if all(isinstance(i, numbers.Integral) for i in values):
                 item.values_int.extend(values)
             elif all(isinstance(i, numbers.Real) for i in values):
@@ -264,13 +323,13 @@ class HandleManager(object):
             result.response.sampled_history_response.item.append(item)
         self._result_q.put(result)
 
-    def handle_request_shutdown(self, record):
+    def handle_request_shutdown(self, record: Record) -> None:
         # TODO(jhr): should we drain things and stop new requests from coming in?
         result = wandb_internal_pb2.Result(uuid=record.uuid)
         self._result_q.put(result)
         self._stopped.set()
 
-    def finish(self):
+    def finish(self) -> None:
         logger.info("shutting down handler")
         if self._tb_watcher:
             self._tb_watcher.finish()
