@@ -3,7 +3,6 @@ from gql import Client, gql  # type: ignore
 from gql.client import RetryError  # type: ignore
 from gql.transport.requests import RequestsHTTPTransport  # type: ignore
 import datetime
-import os
 import ast
 import os
 import json
@@ -12,11 +11,7 @@ import re
 import click
 import logging
 import requests
-import socket
-import time
 import sys
-import random
-import traceback
 
 if os.name == "posix" and sys.version_info[0] < 3:
     import subprocess32 as subprocess  # type: ignore
@@ -24,21 +19,18 @@ else:
     import subprocess  # type: ignore[no-redef]
 
 import six
-from six import b
 from six import BytesIO
 import wandb
 from wandb import __version__
-from wandb.old.core import wandb_dir, Error
 from wandb import env
 from wandb.old.settings import Settings
 from wandb.old import retry
 from wandb import util
 from wandb.apis.normalize import normalize_exceptions
 from wandb.errors.error import CommError, UsageError
-from ..lib.filenames import DIFF_FNAME, METADATA_FNAME
+from ..lib.filenames import DIFF_FNAME
 from ..lib.git import GitRepo
 
-from .file_stream import FileStreamApi
 from .progress import Progress
 
 logger = logging.getLogger(__name__)
@@ -1181,7 +1173,7 @@ class Api(object):
 
         size, response = self.download_file(metadata["url"])
 
-        with open(path, "wb") as file:
+        with util.fsync_open(path, "wb") as file:
             for data in response.iter_content(chunk_size=1024):
                 file.write(data)
 
@@ -1675,6 +1667,7 @@ class Api(object):
         labels=None,
         metadata=None,
         aliases=None,
+        distributed_id=None,
         is_user_created=False,
     ):
         mutation = gql(
@@ -1685,11 +1678,12 @@ class Api(object):
             $entityName: String!,
             $projectName: String!,
             $runName: String,
-            $description: String
+            $description: String,
             $digest: String!,
-            $labels: JSONString
-            $aliases: [ArtifactAliasInput!]
-            $metadata: JSONString
+            $labels: JSONString,
+            $aliases: [ArtifactAliasInput!],
+            $metadata: JSONString,
+            %s
         ) {
             createArtifact(input: {
                 artifactTypeName: $artifactTypeName,
@@ -1700,9 +1694,10 @@ class Api(object):
                 description: $description,
                 digest: $digest,
                 digestAlgorithm: MANIFEST_MD5,
-                labels: $labels
-                aliases: $aliases
-                metadata: $metadata
+                labels: $labels,
+                aliases: $aliases,
+                metadata: $metadata,
+                %s
             }) {
                 artifact {
                     id
@@ -1723,6 +1718,13 @@ class Api(object):
             }
         }
         """
+            %
+            # For backwards compatibility with older backends that don't support
+            # distributed writers.
+            (
+                "$distributedID: String" if distributed_id else "",
+                "distributedID: $distributedID" if distributed_id else "",
+            )
         )
 
         entity_name = entity_name or self.settings("entity")
@@ -1749,6 +1751,7 @@ class Api(object):
                 "metadata": json.dumps(util.make_safe_for_json(metadata))
                 if metadata
                 else None,
+                "distributedID": distributed_id,
             },
         )
         av = response["createArtifact"]["artifact"]
@@ -1794,6 +1797,7 @@ class Api(object):
         project=None,
         run=None,
         include_upload=True,
+        type="FULL",
     ):
         mutation = gql(
             """
@@ -1805,7 +1809,8 @@ class Api(object):
             $entityName: String!,
             $projectName: String!,
             $runName: String!,
-            $includeUpload: Boolean!
+            $includeUpload: Boolean!,
+            %s
         ) {
             createArtifactManifest(input: {
                 name: $name,
@@ -1814,7 +1819,8 @@ class Api(object):
                 baseArtifactID: $baseArtifactID,
                 entityName: $entityName,
                 projectName: $projectName,
-                runName: $runName
+                runName: $runName,
+                %s
             }) {
                 artifactManifest {
                     id
@@ -1829,6 +1835,13 @@ class Api(object):
             }
         }
         """
+            %
+            # For backwards compatibility with older backends that don't support
+            # patch manifests.
+            (
+                "$type: ArtifactManifestType = FULL" if type != "FULL" else "",
+                "type: $type" if type != "FULL" else "",
+            )
         )
 
         entity_name = entity or self.settings("entity")
@@ -1846,10 +1859,64 @@ class Api(object):
                 "projectName": project_name,
                 "runName": run_name,
                 "includeUpload": include_upload,
+                "type": type,
             },
         )
 
-        return response["createArtifactManifest"]["artifactManifest"]["file"]
+        return (
+            response["createArtifactManifest"]["artifactManifest"]["id"],
+            response["createArtifactManifest"]["artifactManifest"]["file"],
+        )
+
+    def update_artifact_manifest(
+        self,
+        artifact_manifest_id,
+        base_artifact_id=None,
+        digest=None,
+        include_upload=True,
+    ):
+        mutation = gql(
+            """
+        mutation UpdateArtifactManifest(
+            $artifactManifestID: ID!,
+            $digest: String,
+            $baseArtifactID: ID,
+            $includeUpload: Boolean!,
+        ) {
+            updateArtifactManifest(input: {
+                artifactManifestID: $artifactManifestID,
+                digest: $digest,
+                baseArtifactID: $baseArtifactID,
+            }) {
+                artifactManifest {
+                    id
+                    file {
+                        id
+                        name
+                        displayName
+                        uploadUrl @include(if: $includeUpload)
+                        uploadHeaders @include(if: $includeUpload)
+                    }
+                }
+            }
+        }
+        """
+        )
+
+        response = self.gql(
+            mutation,
+            variable_values={
+                "artifactManifestID": artifact_manifest_id,
+                "digest": digest,
+                "baseArtifactID": base_artifact_id,
+                "includeUpload": include_upload,
+            },
+        )
+
+        return (
+            response["updateArtifactManifest"]["artifactManifest"]["id"],
+            response["updateArtifactManifest"]["artifactManifest"]["file"],
+        )
 
     @normalize_exceptions
     def create_artifact_files(self, artifact_files):
