@@ -1,25 +1,26 @@
 #
+import contextlib
 import re
 import os
 import time
 import shutil
 import requests
-import threading
 
 from six.moves.urllib.parse import urlparse, quote
 
+import wandb
 from wandb.compat import tempfile as compat_tempfile
-from wandb import env
 from .interface.artifacts import *
-from .internal.progress import Progress
 from wandb.apis import InternalApi, PublicApi
 from wandb.apis.public import Artifact as PublicArtifact
 from wandb.errors.error import CommError
 from wandb import util
 from wandb.errors.term import termwarn, termlog
-from .lib import filesystem
 from wandb.data_types import WBValue
 
+if wandb.TYPE_CHECKING:  # type: ignore
+    from typing import Optional
+
 # This makes the first sleep 1s, and then doubles it up to total times,
 # which makes for ~18 hours.
 _REQUEST_RETRY_STRATEGY = requests.packages.urllib3.util.retry.Retry(
@@ -36,11 +37,15 @@ _REQUEST_POOL_MAXSIZE = 64
 class Artifact(object):
     """An artifact object you can write files into, and pass to log_artifact."""
 
+    name: str
+    description: Optional[str]
+    distributed_id: Optional[str]
+
     def __init__(self, name, type, description=None, metadata=None):
         if not re.match(r"^[a-zA-Z0-9_\-.]+$", name):
             raise ValueError(
-                'Artifact name may only contain alphanumeric characters, dashes, underscores, and dots. Invalid name: "%s"'
-                % name
+                "Artifact name may only contain alphanumeric characters, dashes, underscores, and dots. "
+                'Invalid name: "%s"' % name
             )
         # TODO: this shouldn't be a property of the artifact. It's a more like an
         # argument to log_artifact.
@@ -60,8 +65,8 @@ class Artifact(object):
         self._file_entries = None
         self._manifest = ArtifactManifestV1(self, self._storage_policy)
         self._cache = get_artifacts_cache()
-        self._added_new = False
         self._added_objs = {}
+        self._added_local_paths = {}
         # You can write into this directory when creating artifact files
         self._artifact_dir = compat_tempfile.TemporaryDirectory(
             missing_ok_on_cleanup=True
@@ -70,6 +75,7 @@ class Artifact(object):
         self.name = name
         self.description = description
         self.metadata = metadata
+        self.distributed_id = None
 
     @property
     def id(self):
@@ -100,6 +106,7 @@ class Artifact(object):
         if self._final:
             raise ValueError("Can't add to finalized artifact.")
 
+    @contextlib.contextmanager
     def new_file(self, name, mode="w"):
         self._ensure_can_add()
         path = os.path.join(self._artifact_dir.name, name.lstrip("/"))
@@ -107,9 +114,12 @@ class Artifact(object):
             raise ValueError(
                 'File with name "%s" already exists at "%s"' % (name, path)
             )
+
         util.mkdir_exists_ok(os.path.dirname(path))
-        self._added_new = True
-        return open(path, mode)
+        with util.fsync_open(path, mode) as f:
+            yield f
+
+        self.add_file(path, name=name)
 
     def add_file(self, local_path, name=None, is_tmp=False):
         """Adds a local file to the artifact
@@ -135,16 +145,7 @@ class Artifact(object):
             file_name_parts[0] = b64_string_to_hex(digest)[:8]
             name = os.path.join(file_path, ".".join(file_name_parts))
 
-        entry = ArtifactManifestEntry(
-            name,
-            None,
-            digest=digest,
-            size=os.path.getsize(local_path),
-            local_path=local_path,
-        )
-
-        self._manifest.add_entry(entry)
-        return entry
+        return self._add_local_file(name, local_path, digest=digest)
 
     def add_dir(self, local_path, name=None):
         self._ensure_can_add()
@@ -169,15 +170,7 @@ class Artifact(object):
 
         def add_manifest_file(log_phy_path):
             logical_path, physical_path = log_phy_path
-            self._manifest.add_entry(
-                ArtifactManifestEntry(
-                    logical_path,
-                    None,
-                    digest=md5_file_b64(physical_path),
-                    size=os.path.getsize(physical_path),
-                    local_path=physical_path,
-                )
-            )
+            self._add_local_file(logical_path, physical_path)
 
         import multiprocessing.dummy  # this uses threads
 
@@ -198,6 +191,7 @@ class Artifact(object):
                 Artifact.get_path to store a reference to another artifact's entry.
             name (str) - the path to save
         """
+        self._ensure_can_add()
 
         # This is a bit of a hack, we want to check if the uri is a of the type
         # ArtifactEntry which is a private class returned by Artifact.get_path in
@@ -214,8 +208,7 @@ class Artifact(object):
             raise ValueError(
                 "References must be URIs. To reference a local file, use file://"
             )
-        if self._final:
-            raise ValueError("Can't add to finalized artifact.")
+
         manifest_entries = self._storage_policy.store_reference(
             self, uri, name=name, checksum=checksum, max_objects=max_objects
         )
@@ -232,6 +225,7 @@ class Artifact(object):
             obj (wandb.WBValue): The object to save in an artifact
             name (str): The path to save
         """
+        self._ensure_can_add()
 
         # Validate that the object is wandb.Media type
         if not isinstance(obj, WBValue):
@@ -239,7 +233,7 @@ class Artifact(object):
 
         obj_id = id(obj)
         if obj_id in self._added_objs:
-            return self._added_objs[obj_id]
+            return self._added_objs[obj_id]["entry"]
 
         # If the object is coming from another artifact, save it as a reference
         if obj.artifact_source is not None:
@@ -263,13 +257,13 @@ class Artifact(object):
         # It will be added again later on finalize, but succeed since
         # the checksum should match
         entry = self.add_file(os.path.join(self._artifact_dir.name, name), name)
-        self._added_objs[obj_id] = entry
+        self._added_objs[obj_id] = {"entry": entry, "obj": obj}
 
         return entry
 
     def get_added_local_path_name(self, local_path):
         """If local_path was already added to artifact, return its internal name."""
-        entry = self._manifest.get_entry_by_local_path(local_path)
+        entry = self._added_local_paths.get(local_path, None)
         if entry is None:
             return None
         return entry.path
@@ -287,37 +281,25 @@ class Artifact(object):
         if self._final:
             return self._file_entries
 
-        # Record any created files in the manifest.
-        if self._added_new:
-            self.add_dir(self._artifact_dir.name)
-
         # mark final after all files are added
         self._final = True
         self._digest = self._manifest.digest()
 
-        # If there are new files, move them into the artifact cache now. Our temp
-        # self._artifact_dir may not be available by the time file pusher syncs
-        # these files.
-        if self._added_new:
-            # Update the file entries for new files to point at their new location.
-            def remap_entry(entry):
-                if entry.local_path is None or not entry.local_path.startswith(
-                    self._artifact_dir.name
-                ):
-                    return entry
-                rel_path = os.path.relpath(
-                    entry.local_path, start=self._artifact_dir.name
-                )
-                local_path = os.path.join(self._artifact_dir.name, rel_path)
-                cache_path, hit = self._cache.check_md5_obj_path(
-                    entry.digest, entry.size
-                )
-                if not hit:
-                    shutil.copyfile(local_path, cache_path)
-                entry.local_path = cache_path
+    def _add_local_file(self, name, path, digest=None):
+        digest = digest or md5_file_b64(path)
+        size = os.path.getsize(path)
 
-            for entry in self._manifest.entries.values():
-                remap_entry(entry)
+        cache_path, hit = self._cache.check_md5_obj_path(digest, size)
+        if not hit:
+            shutil.copyfile(path, cache_path)
+
+        entry = ArtifactManifestEntry(
+            name, None, digest=digest, size=size, local_path=cache_path,
+        )
+
+        self._manifest.add_entry(entry)
+        self._added_local_paths[path] = entry
+        return entry
 
 
 class ArtifactManifestV1(ArtifactManifest):
@@ -481,7 +463,7 @@ class WandbStoragePolicy(StoragePolicy):
         )
         response.raise_for_status()
 
-        with open(path, "wb") as file:
+        with util.fsync_open(path, "wb") as file:
             for data in response.iter_content(chunk_size=16 * 1024):
                 file.write(data)
         return path
@@ -516,7 +498,9 @@ class WandbStoragePolicy(StoragePolicy):
         else:
             raise Exception("unrecognized storage layout: {}".format(storage_layout))
 
-    def store_file(self, artifact_id, entry, preparer, progress_callback=None):
+    def store_file(
+        self, artifact_id, artifact_manifest_id, entry, preparer, progress_callback=None
+    ):
         # write-through cache
         cache_path, hit = self._cache.check_md5_obj_path(entry.digest, entry.size)
         if not hit:
@@ -525,6 +509,7 @@ class WandbStoragePolicy(StoragePolicy):
         resp = preparer.prepare(
             lambda: {
                 "artifactID": artifact_id,
+                "artifactManifestID": artifact_manifest_id,
                 "name": entry.path,
                 "md5": entry.digest,
             }
@@ -743,6 +728,8 @@ class LocalFileHandler(StorageHandler):
                         )
                     physical_path = os.path.join(root, sub_path)
                     logical_path = os.path.relpath(physical_path, start=local_path)
+                    if name is not None:
+                        logical_path = os.path.join(name, logical_path)
                     entry = ArtifactManifestEntry(
                         logical_path,
                         os.path.join(path, logical_path),
@@ -1131,7 +1118,7 @@ class HTTPHandler(StorageHandler):
                 % (manifest_entry.ref, manifest_entry.digest, digest)
             )
 
-        with open(path, "wb") as file:
+        with util.fsync_open(path, "wb") as file:
             for data in response.iter_content(chunk_size=16 * 1024):
                 file.write(data)
         return path
