@@ -211,8 +211,9 @@ class _WandbInit(object):
         logger.addHandler(handler)
         # TODO: make me configurable
         logger.setLevel(logging.DEBUG)
-        # TODO: we may need to close the handler as well...
-        self._teardown_hooks.append(lambda: logger.removeHandler(handler))
+        self._teardown_hooks.append(
+            lambda: (handler.close(), logger.removeHandler(handler))
+        )
 
     def _safe_symlink(self, base, target, name, delete=False):
         # TODO(jhr): do this with relpaths, but i cant figure it out on no sleep
@@ -319,10 +320,17 @@ class _WandbInit(object):
         logger.info("Logging internal logs to {}".format(settings.log_internal))
 
     def init(self) -> Union[Run, Dummy, None]:  # noqa: C901
+        assert logger
+        logger.info("calling init triggers")
         trigger.call("on_init", **self.kwargs)
         s = self.settings
         sweep_config = self.sweep_config
         config = self.config
+        logger.info(
+            "wandb.init called with sweep_config: {}\nconfig: {}".format(
+                sweep_config, config
+            )
+        )
         if s._noop:
             drun = Dummy()
             drun.config = wandb.wandb_sdk.wandb_config.Config()
@@ -357,6 +365,11 @@ class _WandbInit(object):
                     )
 
                 last_id = self._wl._global_run_stack[-1]._run_id
+                logger.info(
+                    "re-initializing run, found existing run on stack: {}".format(
+                        last_id
+                    )
+                )
                 jupyter = (
                     s._jupyter
                     and not s._silent
@@ -385,6 +398,7 @@ class _WandbInit(object):
         stdout_master_fd, stderr_master_fd = None, None
         stdout_slave_fd, stderr_slave_fd = None, None
 
+        logger.info("starting backend")
         backend = Backend()
         backend.ensure_launched(
             settings=s,
@@ -393,6 +407,7 @@ class _WandbInit(object):
             use_redirect=use_redirect,
         )
         backend.server_connect()
+        logger.info("backend started and connected")
         # Make sure we are logged in
         # wandb_login._login(_backend=backend, _settings=self.settings)
 
@@ -415,6 +430,7 @@ class _WandbInit(object):
                 tel.env.windows = True
             run._telemetry_imports(tel.imports_init)
 
+        logger.info("updated telemetry")
         run._set_console(
             use_redirect=use_redirect,
             stdout_slave_fd=stdout_slave_fd,
@@ -437,10 +453,12 @@ class _WandbInit(object):
             backend.interface._publish_run(run_proto)
             run._set_run_obj_offline(run_proto)
         else:
+            logger.info("communicating current version")
             ret = backend.interface.communicate_check_version(
                 current_version=wandb.__version__
             )
             if ret:
+                logger.info("got version response {}".format(ret))
                 if ret.upgrade_message:
                     run._set_upgraded_version_message(ret.upgrade_message)
                 if ret.delete_message:
@@ -448,25 +466,33 @@ class _WandbInit(object):
                 if ret.yank_message:
                     run._set_yanked_version_message(ret.yank_message)
             run._on_init()
+            logger.info("communicating run to backend with 30 second timeout")
             ret = backend.interface.communicate_run(run, timeout=30)
             error_message = None
             if not ret:
-                error_message = "Error communicating with backend"
+                logger.error("backend process timed out")
+                error_message = "Error communicating with wandb process"
+                if self.settings.start_method != "fork":
+                    error_message += ", try setting WANDB_START_METHOD=fork"
             if ret and ret.error:
                 error_message = ret.error.message
             if error_message:
+                logger.error("encountered error: {}".format(error_message))
                 # Shutdown the backend and get rid of the logger
                 # we don't need to do console cleanup at this point
                 backend.cleanup()
                 self.teardown()
                 raise UsageError(error_message)
             if ret.run.resumed:
+                logger.info("run resumed")
                 with telemetry.context(run=run) as tel:
                     tel.feature.resumed = True
             run._set_run_obj(ret.run)
 
+        logger.info("starting run threads in backend")
         # initiate run (stats and metadata probing)
-        _ = backend.interface.communicate_run_start()
+        run_obj = run._run_obj or run._run_obj_offline
+        _ = backend.interface.communicate_run_start(run_obj)
 
         self._wl._global_run_stack.append(run)
         self.run = run
@@ -486,6 +512,7 @@ class _WandbInit(object):
         run._on_start()
 
         run._freeze()
+        logger.info("run started, returning control to user process")
         return run
 
 
@@ -522,69 +549,140 @@ def init(
     id=None,
     settings: Union[Settings, Dict[str, Any], None] = None,
 ) -> Union[Run, Dummy, None]:
-    """Initialize W&B
-    Spawns a new process to start or resume a run locally and communicate with a
-    wandb server. Should be called before any calls to wandb.log.
+    """
+    Start a new tracked run with `wandb.init()`.
+
+    In an ML training pipeline, you could add `wandb.init()`
+    to the beginning of your training script as well as your evaluation
+    script, and each piece would be tracked as a run in W&B.
+
+    `wandb.init()` spawns a new background process to log data to a run, and it
+    also syncs data to wandb.ai by default so you can see live visualizations.
+    Call `wandb.init()` to start a run before logging data with `wandb.log()`.
+
+    `wandb.init()` returns a run object, and you can also access the run object
+    with wandb.run.
 
     Arguments:
-        job_type (str, optional): The type of job running, defaults to 'train'
-        dir (str, optional): An absolute path to a directory where metadata will
-            be stored.
-        config (dict, argparse, or absl.flags, str, optional):
-            Sets the config parameters (typically hyperparameters) to store with the
-            run. See also wandb.config.
+        project: (str, optional) The name of the project where you're sending
+            the new run. If the project is not specified, the run is put in an
+            "Uncategorized" project.
+        entity: (str, optional) An entity is a username or team name where
+            you're sending runs. This entity must exist before you can send runs
+            there, so make sure to create your account or team in the UI before
+            starting to log runs.
+            If you don't specify an entity, the run will be sent to your default
+            entity, which is usually your username. Change your default entity
+            in [Settings](wandb.ai/settings) under "default location to create
+            new projects".
+        config: (dict, argparse, absl.flags, str, optional)
+            This sets wandb.config, a dictionary-like object for saving inputs
+            to your job, like hyperparameters for a model or settings for a data
+            preprocessing job. The config will show up in a table in the UI that
+            you can use to group, filter, and sort runs. Keys should not contain
+            `.` in their names, and values should be under 10 MB.
             If dict, argparse or absl.flags: will load the key value pairs into
-                the runs config object.
-            If str: will look for a yaml file that includes config parameters and
-                load them into the run's config object.
-        project (str, optional): W&B Project.
-        entity (str, optional): W&B Entity.
-        reinit (bool, optional): Allow multiple calls to init in the same process.
-        tags (list, optional): A list of tags to apply to the run.
-        group (str, optional): A unique string shared by all runs in a given group.
-        name (str, optional): A display name for the run which does not have to be
-            unique.
-        notes (str, optional): A multiline string associated with the run.
-        magic (bool, dict, or str, optional): magic configuration as bool, dict,
-            json string, yaml filename.
-        config_exclude_keys (list, optional): string keys to exclude storing in W&B
-            when specifying config.
-        config_include_keys (list, optional): string keys to include storing in W&B
-            when specifying config.
-        anonymous (str, optional): Can be "allow", "must", or "never". Controls
-            whether anonymous logging is allowed.  Defaults to never.
-        mode (str, optional): Can be "online", "offline" or "disabled". Defaults to
-            online.
-        allow_val_change (bool, optional): allow config values to be changed after
-            setting. Defaults to true in jupyter and false otherwise.
-        resume (bool, str, optional): Sets the resuming behavior. Should be one of:
+                the wandb.config object.
+            If str: will look for a yaml file by that name, and load config from
+                that file into the wandb.config object.
+        save_code: (bool, optional) Turn this on to save the main script or
+            notebook to W&B. This is valuable for improving experiment
+            reproducibility and to diff code across experiments in the UI. By
+            default this is off, but you can flip the default behavior to "on"
+            in [Settings](wandb.ai/settings).
+        group: (str, optional) Specify a group to organize individual runs into
+            a larger experiment. For example, you might be doing cross
+            validation, or you might have multiple jobs that train and evaluate
+            a model against different test sets. Group gives you a way to
+            organize runs together into a larger whole, and you can toggle this
+            on and off in the UI. For more details, see
+            [Grouping](docs.wandb.com/library/grouping).
+        job_type: (str, optional) Specify the type of run, which is useful when
+            you're grouping runs together into larger experiments using group.
+            For example, you might have multiple jobs in a group, with job types
+            like train and eval. Setting this makes it easy to filter and group
+            similar runs together in the UI so you can compare apples to apples.
+        tags: (list, optional) A list of strings, which will populate the list
+            of tags on this run in the UI. Tags are useful for organizing runs
+            together, or applying temporary labels like "baseline" or
+            "production". It's easy to add and remove tags in the UI, or filter
+            down to just runs with a specific tag.
+        name: (str, optional) A short display name for this run, which is how
+            you'll identify this run in the UI. By default we generate a random
+            two-word name that lets you easily cross-reference runs from the
+            table to charts. Keeping these run names short makes the chart
+            legends and tables easier to read. If you're looking for a place to
+            save your hyperparameters, we recommend saving those in config.
+        notes: (str, optional) A longer description of the run, like a -m commit
+            message in git. This helps you remember what you were doing when you
+            ran this run.
+        dir: (str, optional) An absolute path to a directory where metadata will
+            be stored. When you call download() on an artifact, this is the
+            directory where downloaded files will be saved. By default this is
+            the ./wandb directory.
+        sync_tensorboard: (bool, optional) Whether to copy all TensorBoard logs
+            to W&B (default: False).
+            [Tensorboard](https://docs.wandb.com/integrations/tensorboard)
+        resume (bool, str, optional): Sets the resuming behavior. Options:
             "allow", "must", "never", "auto" or None. Defaults to None.
             Cases:
-            - "auto" (or True): automatically resume the previous run on the same machine.
-                if the previous run crashed, otherwise starts a new run.
-            - "allow": if id is set with init(id="UNIQUE_ID") or WANDB_RUN_ID="UNIQUE_ID"
-                and it is identical to a previous run, wandb will automatically resume the
-                run with the id. Otherwise wandb will start a new run.
-            - "never": if id is set with init(id="UNIQUE_ID") or WANDB_RUN_ID="UNIQUE_ID"
-                and it is identical to a previous run, wandb will crash.
-            - "must": if id is set with init(id="UNIQUE_ID") or WANDB_RUN_ID="UNIQUE_ID"
-                and it is identical to a previous run, wandb will automatically resume the
-                run with the id. Otherwise wandb will crash.
-            - None: never resumes - if a run has a duplicate run_id the previous run is
-                overwritten.
-            See https://docs.wandb.com/library/advanced/resuming for more detail.
-        force (bool, optional): If true, will cause script to crash if user can't or isn't
-            logged in to a wandb server.  If false, will cause script to run in offline
-            modes if user can't or isn't logged in to a wandb server. Defaults to false.
-        sync_tensorboard (bool, optional): Synchronize wandb logs from tensorboard or
+            - None (default): If the new run has the same ID as a previous run,
+                this run overwrites that data.
+            - "auto" (or True): if the preivous run on this machine crashed,
+                automatically resume it. Otherwise, start a new run.
+            - "allow": if id is set with init(id="UNIQUE_ID") or
+                WANDB_RUN_ID="UNIQUE_ID" and it is identical to a previous run,
+                wandb will automatically resume the run with that id. Otherwise,
+                wandb will start a new run.
+            - "never": if id is set with init(id="UNIQUE_ID") or
+                WANDB_RUN_ID="UNIQUE_ID" and it is identical to a previous run,
+                wandb will crash.
+            - "must": if id is set with init(id="UNIQUE_ID") or
+                WANDB_RUN_ID="UNIQUE_ID" and it is identical to a previous run,
+                wandb will automatically resume the run with the id. Otherwise
+                wandb will crash.
+            See https://docs.wandb.com/library/advanced/resuming for more.
+        reinit: (bool, optional) Allow multiple wandb.init() calls in the same
+            process. (default: False)
+        magic: (bool, dict, or str, optional) The bool controls whether we try to
+            auto-instrument your script, capturing basic details of your run
+            without you having to add more wandb code. (default: False)
+            You can also pass a dict, json string, or yaml filename.
+        config_exclude_keys: (list, optional) string keys to exclude from
+            `wandb.config`.
+        config_include_keys: (list, optional) string keys to include in
+            wandb.config.
+        anonymous: (str, optional) Controls anonymous data logging. Options:
+            - "never" (default): requires you to link your W&B account before
+                tracking the run so you don't accidentally create an anonymous
+                run.
+            - "allow": lets a logged-in user track runs with their account, but
+                lets someone who is running the script without a W&B account see
+                the charts in the UI.
+            - "must": sends the run to an anonymous account instead of to a
+                signed-up user account.
+        mode: (str, optional) Can be "online", "offline" or "disabled". Defaults to
+            online.
+        allow_val_change: (bool, optional) Whether to allow config values to
+            change after setting the keys once. By default we throw an exception
+            if a config value is overwritten. If you want to track something
+            like a varying learning_rate at multiple times during training, use
+            wandb.log() instead. (default: False in scripts, True in Jupyter)
+        force: (bool, optional) If True, this crashes the script if a user isn't
+            logged in to W&B. If False, this will let the script run in offline
+            mode if a user isn't logged in to W&B. (default: False)
+        sync_tensorboard: (bool, optional) Synchronize wandb logs from tensorboard or
             tensorboardX and saves the relevant events file. Defaults to false.
-        monitor_gym: (bool, optional): automatically logs videos of environment when
-            using OpenAI Gym (see https://docs.wandb.com/library/integrations/openai-gym)
-            Defaults to false.
-        save_code (bool, optional): Save the entrypoint or jupyter session history
-            source code.
-        id (str, optional): A globally unique (per project) identifier for the run. This
-            is primarily used for resuming.
+        monitor_gym: (bool, optional) automatically logs videos of environment when
+            using OpenAI Gym. (default: False)
+            See https://docs.wandb.com/library/integrations/openai-gym
+        id: (str, optional) A unique ID for this run, used for Resuming. It must
+            be unique in the project, and if you delete a run you can't reuse
+            the ID. Use the name field for a short descriptive name, or config
+            for saving hyperparameters to compare across runs. The ID cannot
+            contain special characters.
+            See https://docs.wandb.com/library/resuming
+
 
     Examples:
         Basic usage
