@@ -20,16 +20,17 @@ if wandb.TYPE_CHECKING:  # type: ignore
 
 
 def login(anonymous=None, key=None, relogin=None, host=None, force=None):
-    """Log in to W&B.
+    """
+    Log in to W&B.
 
     Arguments:
-        anonymous (string, optional): Can be "must", "allow", or "never".
+        anonymous: (string, optional) Can be "must", "allow", or "never".
             If set to "must" we'll always login anonymously, if set to
             "allow" we'll only create an anonymous user if the user
             isn't already logged in.
-        key (string, optional): authentication key.
-        relogin (bool, optional): If true, will re-prompt for API key.
-        host (string, optional): The host to connect to.
+        key: (string, optional) authentication key.
+        relogin: (bool, optional) If true, will re-prompt for API key.
+        host: (string, optional) The host to connect to.
 
     Returns:
         bool: if key is configured
