@@ -523,21 +523,23 @@ def run_agent(
 
 
 def agent(sweep_id, function=None, entity=None, project=None, count=None):
-    """Generic agent entrypoint, used for CLI or jupyter.
+    """
+    Generic agent entrypoint, used for CLI or jupyter.
 
     Will run a function or program with configuration parameters specified
-        by server.
+    by server.
 
     Arguments:
-        sweep_id (dict): Sweep ID generated by CLI or sweep API
-        function (func, optional): A function to call instead of the "program"
+        sweep_id: (dict) Sweep ID generated by CLI or sweep API
+        function: (func, optional) A function to call instead of the "program"
             specifed in the config.
-        entity (str, optional): W&B Entity
-        project (str, optional): W&B Project
-        count (int, optional): the number of trials to run.
+        entity: (str, optional) W&B Entity
+        project: (str, optional) W&B Project
+        count: (int, optional) the number of trials to run.
 
     Examples:
         Run a sample sweep over a function:
+        ```
         def train():
             with wandb.init() as run:
                 print("config:", dict(run.config))
@@ -547,6 +549,7 @@ def agent(sweep_id, function=None, entity=None, project=None, count=None):
                     time.sleep(1)
 
         wandb.agent(sweep_id, function=train)
+        ```
     """
     global _INSTANCES
     _INSTANCES += 1
